@@ -81,7 +81,9 @@ def main(argv=None):
     parser.add_option('-d', '--database', dest='db', metavar='<dbname>')
     parser.add_option('-U', '--user', dest='user', metavar='<user>')
     parser.add_option('--proj', dest='proj', metavar='EPSG:900913')
-    parser.add_option('--connection', dest='connection')
+    parser.add_option('--connection', dest='connection',
+        help="connection string like postgis://user:pass@host:port/database,"
+             " this overwrites the -h/-p/-d/-U options")
     
     parser.add_option('-c', '--concurrency', dest='concurrency', metavar='N',
                       type='int', default=n_cpu)
@@ -182,8 +184,8 @@ def main(argv=None):
                 from getpass import getpass
                 db_conf.password = getpass('password for %(user)s at %(host)s:' % db_conf)
         
-            if options.proj:
-                db_conf.proj = options.proj
+        if options.proj:
+            db_conf.proj = options.proj
     
     logger = imposm.util.ProgressLog
     
@@ -208,13 +210,16 @@ def main(argv=None):
     if options.read:
         read_timer = imposm.util.Timer('reading', logger)
         
-        if args:
-            reader = ImposmReader(tag_mapping, cache=cache, merge=options.merge_cache,
-                pool_size=options.concurrency, logger=logger)
-            reader.estimated_coords = imposm.util.estimate_records(args)
-            for arg in args:
-                logger.message('## reading %s' % arg)
-                reader.read(arg)
+        if not args:
+            print "no file(s) supplied"
+            sys.exit(2)
+
+        reader = ImposmReader(tag_mapping, cache=cache, merge=options.merge_cache,
+            pool_size=options.concurrency, logger=logger)
+        reader.estimated_coords = imposm.util.estimate_records(args)
+        for arg in args:
+            logger.message('## reading %s' % arg)
+            reader.read(arg)
         read_timer.stop()
 
     if options.write:
