@@ -20,6 +20,7 @@ import psycopg2.extensions
 import logging
 log = logging.getLogger(__name__)
 
+from imposm import config
 from imposm.mapping import UnionView, GeneralizedTable, Mapping
 
 class PostGISDB(object):
@@ -116,8 +117,8 @@ class PostGISDB(object):
             extra_args = ', %s' * len(extra_arg_names)
             extra_arg_names = ', ' + ', '.join('"' + name + '"' for name in extra_arg_names)
         return """INSERT INTO "%(tablename)s"
-            (osm_id, type, geometry %(extra_arg_names)s)
-            VALUES (%%s, %%s, ST_Transform(ST_GeomFromWKB(%%s, 4326), %(srid)s)
+            (osm_id, geometry %(extra_arg_names)s)
+            VALUES (%%s, ST_Transform(ST_GeomFromWKB(%%s, 4326), %(srid)s)
                 %(extra_args)s)
         """.strip() % dict(tablename=self.table_prefix + mapping.name, srid=self.srid,
             extra_arg_names=extra_arg_names, extra_args=extra_args)
@@ -140,18 +141,30 @@ class PostGISDB(object):
         for n, t in mapping.fields:
             extra_fields += ', "%s" %s ' % (n, t.column_type)
 
+        if config.imposm_pg_serial_id:
+            serial_column = "id SERIAL PRIMARY KEY,"
+        else:
+            serial_column = ""
+            
         cur.execute("""
             CREATE TABLE "%s" (
-                osm_id INT4 PRIMARY KEY,
-                type VARCHAR(255)
+                %s
+                osm_id BIGINT
                 %s
             );
-        """ % (tablename, extra_fields))
+        """ % (tablename, serial_column, extra_fields))
         cur.execute("""
             SELECT AddGeometryColumn ('', '%(tablename)s', 'geometry',
                                       %(srid)s, '%(pg_geometry_type)s', 2)
         """ % dict(tablename=tablename, srid=self.srid,
                    pg_geometry_type=mapping.geom_type))
+
+        for n, t in mapping.fields:
+            if isinstance(t, TrigramIndex):
+                cur.execute("""
+                    CREATE INDEX "%(tablename)s_trgm_idx_%(column)s" ON "%(tablename)s" USING GIST ("%(column)s" gist_trgm_ops)
+                """ % dict(tablename=tablename, column=n))
+
         cur.execute("""
             CREATE INDEX "%(tablename)s_geom" ON "%(tablename)s" USING GIST (geometry)
         """ % dict(tablename=tablename))
@@ -166,15 +179,25 @@ class PostGISDB(object):
         for row in cur:
             table_name = row[0]
             if table_name.startswith(existing_prefix) and not table_name.startswith((new_prefix, backup_prefix)):
+                # check for overlapping prefixes: osm_ but not osm_new_ or osm_backup_
                 existing_tables.append(table_name)
 
         cur.execute('SELECT indexname FROM pg_indexes WHERE indexname like %s', (existing_prefix + '%', ))
         existing_indexes = set()
         for row in cur:
             index_name = row[0]
-            if table_name.startswith(existing_prefix) and not index_name.startswith((new_prefix, backup_prefix)):
+            if index_name.startswith(existing_prefix) and not index_name.startswith((new_prefix, backup_prefix)):
+                # check for overlapping prefixes: osm_ but not osm_new_ or osm_backup_
                 existing_indexes.add(index_name)
         
+        cur.execute('SELECT relname FROM pg_class WHERE relname like %s', (existing_prefix + '%_id_seq', ))
+        existing_seq = set()
+        for row in cur:
+            seq_name = row[0]
+            if seq_name.startswith(existing_prefix) and not seq_name.startswith((new_prefix, backup_prefix)):
+                # check for overlapping prefixes: osm_ but not osm_new_ or osm_backup_
+                existing_seq.add(seq_name)
+
         cur.execute('SELECT tablename FROM pg_tables WHERE tablename like %s', (new_prefix + '%', ))
         new_tables = []
         for row in cur:
@@ -187,25 +210,40 @@ class PostGISDB(object):
             index_name = row[0]
             new_indexes.add(index_name)
         
+        cur.execute('SELECT relname FROM pg_class WHERE relname like %s', (new_prefix + '%_id_seq', ))
+        new_seq = []
+        for row in cur:
+            seq_name = row[0]
+            new_seq.append(seq_name)
+
+        
         if not new_tables:
             raise RuntimeError('did not found tables to swap')
         
+        # rename existing tables (osm_) to backup_prefix (osm_backup_)
         for table_name in existing_tables:
             rename_to = table_name.replace(existing_prefix, backup_prefix)
             cur.execute('ALTER TABLE "%s" RENAME TO "%s"' % (table_name, rename_to))
-            if table_name + '_geom' in existing_indexes:
-                cur.execute('ALTER INDEX "%s" RENAME TO "%s"' % (table_name + '_geom', rename_to + '_geom'))
-            if table_name + '_pkey' in existing_indexes:
-                cur.execute('ALTER INDEX "%s" RENAME TO "%s"' % (table_name + '_pkey', rename_to + '_pkey'))
+
+            for idx in existing_indexes:
+                if idx in (table_name + '_geom', table_name + '_pkey') or idx.startswith(table_name + '_trgm_idx_'):
+                    new_idx = idx.replace(table_name, rename_to, 1)
+                    cur.execute('ALTER INDEX "%s" RENAME TO "%s"' % (idx, new_idx))
+            if table_name + '_id_seq' in existing_seq:
+                cur.execute('ALTER SEQUENCE "%s" RENAME TO "%s"' % (table_name + '_id_seq', rename_to + '_id_seq'))
             cur.execute('UPDATE geometry_columns SET f_table_name = %s WHERE f_table_name = %s', (rename_to, table_name))
             
+        # rename new tables (osm_new_) to existing_prefix (osm_)
         for table_name in new_tables:
             rename_to = table_name.replace(new_prefix, existing_prefix)
             cur.execute('ALTER TABLE "%s" RENAME TO "%s"' % (table_name, rename_to))
-            if table_name + '_geom' in new_indexes:
-                cur.execute('ALTER INDEX "%s" RENAME TO "%s"' % (table_name + '_geom', rename_to + '_geom'))
-            if table_name + '_pkey' in new_indexes:
-                cur.execute('ALTER INDEX "%s" RENAME TO "%s"' % (table_name + '_pkey', rename_to + '_pkey'))
+
+            for idx in new_indexes:
+                if idx in (table_name + '_geom', table_name + '_pkey') or idx.startswith(table_name + '_trgm_idx_'):
+                    new_idx = idx.replace(table_name, rename_to, 1)
+                    cur.execute('ALTER INDEX "%s" RENAME TO "%s"' % (idx, new_idx))
+            if table_name + '_id_seq' in new_seq:
+                cur.execute('ALTER SEQUENCE "%s" RENAME TO "%s"' % (table_name + '_id_seq', rename_to + '_id_seq'))
             cur.execute('UPDATE geometry_columns SET f_table_name = %s WHERE f_table_name = %s', (rename_to, table_name))
         
     def remove_tables(self, prefix):
@@ -269,11 +307,17 @@ class PostGISUnionView(object):
 
     def _view_stmt(self):
         selects = []
+        if config.imposm_pg_serial_id:
+            serial_column = "id, "
+        else:
+            serial_column = ""
+
         for mapping in self.mapping.mappings:
             field_str = ', '.join(self._mapping_fields(mapping))
-            selects.append("""SELECT osm_id, type, geometry, %s,
+            selects.append("""SELECT %s osm_id, geometry, %s,
                 '%s' as class from "%s" """ % (
-                field_str, mapping.classname or mapping.name, self.db.to_tablename(mapping.name)))
+                serial_column, field_str,
+                mapping.classname or mapping.name, self.db.to_tablename(mapping.name)))
 
         selects = '\nUNION ALL\n'.join(selects)
 
@@ -346,9 +390,16 @@ class PostGISGeneralizedTable(object):
             where = ' WHERE ' + self.mapping.where
         else:
             where = ''
-        return """CREATE TABLE "%s" AS (SELECT osm_id, type, %s
+        
+        if config.imposm_pg_serial_id:
+            serial_column = "id, "
+        else:
+            serial_column = ""
+    
+        return """CREATE TABLE "%s" AS (SELECT %s osm_id, %s
             ST_Simplify(geometry, %f) as geometry from "%s"%s)""" % (
-            self.table_name, fields, self.mapping.tolerance, self.db.to_tablename(self.mapping.origin.name),
+            self.table_name, serial_column, fields, self.mapping.tolerance,
+            self.db.to_tablename(self.mapping.origin.name),
             where)
 
     def create(self):
@@ -369,3 +420,5 @@ class PostGISGeneralizedTable(object):
             cur.execute('DELETE FROM geometry_columns WHERE f_table_name = %s', (self.table_name, ))
         cur.execute(self._geom_table_stmt())
 
+class TrigramIndex(object):
+    pass
