@@ -12,11 +12,17 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from __future__ import division
+
+import math
+
 import shapely.geos
 from shapely.geometry.base import BaseGeometry
 from shapely import geometry
 from shapely import wkt
 
+from imposm import config
+
 import logging
 log = logging.getLogger(__name__)
 
@@ -35,13 +41,22 @@ SHAPELY_SUPPORTS_BUFFER = shapely.geos.geos_capi_version >= (1, 6, 0)
 
 def validate_and_simplify(geom, meter_units=False):
     if SHAPELY_SUPPORTS_BUFFER:
-        # buffer(0) is nearly fast as is_valid 
-        return geom.buffer(0)
+        try:
+            # buffer(0) is nearly fast as is_valid 
+            return geom.buffer(0)
+        except ValueError:
+            # shapely raises ValueError if buffer(0) result is empty
+            raise InvalidGeometryError('geometry is empty')
     
     orig_geom = geom
     if not geom.is_valid:
         tolerance = TOLERANCE_METERS if meter_units else TOLERANCE_DEEGREES
-        geom = geom.simplify(tolerance, False)
+        try:
+            geom = geom.simplify(tolerance, False)
+        except ValueError:
+            # shapely raises ValueError if buffer(0) result is empty
+            raise InvalidGeometryError('geometry is empty')
+
         if not geom.is_valid:
             raise InvalidGeometryError('geometry is invalid, could not simplify: %s' %
                                        orig_geom)
@@ -143,11 +158,20 @@ class LineStringBuilder(GeomBuilder):
         if geom.type != 'LineString':
             raise InvalidGeometryError('expected LineString, got %s' % geom.type)
 
-    def to_geom(self, data):
+    def to_geom(self, data, max_length=None):
         if len(data) <= 1:
             return None
         if len(data) == 2 and data[0] == data[1]:
             return None
+        if max_length is None:
+            max_length = config.imposm_linestring_max_length
+        if max_length and len(data) > max_length:
+            chunks = math.ceil(len(data) / max_length)
+            length = int(len(data) // chunks)
+            lines = []
+            for i in xrange(1, len(data), length):
+                lines.append(geometry.LineString(data[i-1:i+length]))
+            return lines
         return geometry.LineString(data)
 
     def build_checked_geom(self, osm_elem, validate=False):
