@@ -84,11 +84,17 @@ class Mapping(object):
     field_filter = ()
     classname = None
     _insert_stmt = None
-    
-    def __init__(self, name, mapping, fields=None, field_filter=None):
+    with_type_field = True
+
+    def __init__(self, name, mapping, fields=None, field_filter=None, with_type_field=None):
         self.name = name
         self.mapping = mapping
         self.fields = fields or tuple(self.fields)
+        if with_type_field is not None:
+            # allow subclass to define other default by setting it as class variable
+            self.with_type_field = with_type_field
+        if self.with_type_field:
+            self._add_type_field()
         self._add_name_field()
         if field_filter:
             self.field_filter = field_filter
@@ -103,6 +109,12 @@ class Mapping(object):
             else:
                 self.fields = (('name', Name()),) + self.fields
         
+    def _add_type_field(self):
+        """
+        Add type field.
+        """
+        self.fields = (('type', Type()), ) + self.fields
+
     @property
     def insert_stmt(self):
         if not self._insert_stmt:
@@ -276,7 +288,7 @@ class Points(Mapping):
     """
     Table class for point features.
     
-    :PostGIS datatype: POINT (for multi-polygon support)
+    :PostGIS datatype: POINT
     """
     table = PointTable
     geom_builder = imposm.geom.PointBuilder()
@@ -286,7 +298,7 @@ class LineStrings(Mapping):
     """
     Table class for line string features.
     
-    :PostGIS datatype: LINESTRING (for multi-polygon support)
+    :PostGIS datatype: LINESTRING
     """
     table = LineStringTable
     geom_builder = imposm.geom.LineStringBuilder()
@@ -296,7 +308,7 @@ class Polygons(Mapping):
     """
     Table class for polygon features.
     
-    :PostGIS datatype: GEOMETRY (for multi-polygon support)
+    :PostGIS datatype: GEOMETRY (POLYGON does not support multi-polygons)
     """
     table = PolygonTable
     geom_builder = imposm.geom.PolygonBuilder()
@@ -310,6 +322,7 @@ class GeneralizedTable(object):
         self.origin = origin
         self.classname = origin.name
         self.fields = self.origin.fields
+        self.with_type_field = self.origin.with_type_field
         self.where = where
 
 class UnionView(object):
@@ -318,7 +331,8 @@ class UnionView(object):
         self.mappings = mappings
         self.fields = fields
         self._add_name_field()
-        
+        self._add_type_field()
+
     def _add_name_field(self):
         """
         Add name field to default if not set.
@@ -329,6 +343,14 @@ class UnionView(object):
             else:
                 self.fields = (('name', ''),) + self.fields
 
+    def _add_type_field(self):
+        """
+        Add type field if not configured and at least one mapping has a type field.
+        """
+        if 'type' not in self.fields and any(m.with_type_field for m in self.mappings):
+            self.fields += (('type', None), )
+
+        
 class DropElem(Exception):
     pass
 
@@ -347,6 +369,83 @@ class FieldType(object):
     def value(self, val, osm_elem):
         return val
 
+class Type(FieldType):
+    """
+    Field for type values (i.e. the *value* of the mapped key/value).
+
+    Use this in combination with ``with_type_field=False`` of the
+    mapping class, if you want to store the value of the mapped
+    key/value in a different column.
+
+    For example, to get a column ``road_class`` instead of ``type``::
+
+        roads = LineStrings(
+            with_type_field = False,
+            name = 'roads',
+            mapping = {
+                'highway': (
+                    'motorway',
+                    'trunk',
+                    'secondary',
+                ),
+            },
+            fields = (
+                ('road_class', Type(),),
+            ),
+        )
+    
+    :PostgreSQL datatype: VARCHAR(255)
+
+    .. versionadded:: 2.4.0
+
+    """
+    column_type = "VARCHAR(255)"
+
+    def extra_fields(self):
+        return []
+
+    def value(self, val, osm_elem):
+        return osm_elem.type
+
+class Class(FieldType):
+    """
+    Field for class values (i.e. the *key* of the mapped key/value).
+
+    Use this if you want to store the key that was used for 
+    this mapping. For example, the following mapping will
+    create a column ``class`` that will have the value
+    ``landuse`` or ``natural``, depending on the feature.
+    
+    ::
+
+        landusages = Polygons(
+            name = 'landusages',
+            fields = (
+                ('class', Class()),
+            ),
+            mapping = {
+                'landuse': (
+                    'wood',
+                ),
+                'natural': (
+                    'wood',
+                ),
+            }
+        )
+
+    :PostgreSQL datatype: VARCHAR(255)
+    
+    .. versionadded:: 2.4.0
+
+    """
+    column_type = "VARCHAR(255)"
+
+    def extra_fields(self):
+        return []
+
+    def value(self, val, osm_elem):
+        return osm_elem.cls
+
 class String(FieldType):
     """
     Field for string values.
